@@ -75,6 +75,7 @@ impl<T, N> Borrow<[T]> for GenericArray<T, N>
 where
     N: ArrayLength<T>,
 {
+    #[inline(always)]
     fn borrow(&self) -> &[T] {
         &self[..]
     }
@@ -84,6 +85,7 @@ impl<T, N> BorrowMut<[T]> for GenericArray<T, N>
 where
     N: ArrayLength<T>,
 {
+    #[inline(always)]
     fn borrow_mut(&mut self) -> &mut [T] {
         &mut self[..]
     }
@@ -93,6 +95,7 @@ impl<T, N> AsRef<[T]> for GenericArray<T, N>
 where
     N: ArrayLength<T>,
 {
+    #[inline(always)]
     fn as_ref(&self) -> &[T] {
         &self[..]
     }
@@ -102,6 +105,7 @@ impl<T, N> AsMut<[T]> for GenericArray<T, N>
 where
     N: ArrayLength<T>,
 {
+    #[inline(always)]
     fn as_mut(&mut self) -> &mut [T] {
         &mut self[..]
     }
@@ -123,15 +127,33 @@ macro_rules! impl_from {
     ($($n: expr => $ty: ty),*) => {
         $(
             impl<T> From<[T; $n]> for GenericArray<T, $ty> {
+                #[inline(always)]
                 fn from(arr: [T; $n]) -> Self {
-                    use core::mem::{forget, transmute_copy};
-                    let x = unsafe { transmute_copy(&arr) };
-                    forget(arr);
-                    x
+                    unsafe { $crate::transmute(arr) }
+                }
+            }
+
+            impl<T> From<GenericArray<T, $ty>> for [T; $n] {
+                #[inline(always)]
+                fn from(sel: GenericArray<T, $ty>) -> [T; $n] {
+                    unsafe { $crate::transmute(sel) }
+                }
+            }
+
+            impl<T> AsRef<[T; $n]> for GenericArray<T, $ty> {
+                #[inline]
+                fn as_ref(&self) -> &[T; $n] {
+                    unsafe { $crate::transmute(self) }
+                }
+            }
+
+            impl<T> AsMut<[T; $n]> for GenericArray<T, $ty> {
+                #[inline]
+                fn as_mut(&mut self) -> &mut [T; $n] {
+                    unsafe { $crate::transmute(self) }
                 }
             }
         )*
-
     }
 }
 
@@ -169,3 +191,56 @@ impl_from! {
     31 => ::typenum::U31,
     32 => ::typenum::U32
 }
+
+#[cfg(feature = "more_lengths")]
+impl_from! {
+    33 => ::typenum::U33,
+    34 => ::typenum::U34,
+    35 => ::typenum::U35,
+    36 => ::typenum::U36,
+    37 => ::typenum::U37,
+    38 => ::typenum::U38,
+    39 => ::typenum::U39,
+    40 => ::typenum::U40,
+    41 => ::typenum::U41,
+    42 => ::typenum::U42,
+    43 => ::typenum::U43,
+    44 => ::typenum::U44,
+    45 => ::typenum::U45,
+    46 => ::typenum::U46,
+    47 => ::typenum::U47,
+    48 => ::typenum::U48,
+    49 => ::typenum::U49,
+    50 => ::typenum::U50,
+    51 => ::typenum::U51,
+    52 => ::typenum::U52,
+    53 => ::typenum::U53,
+    54 => ::typenum::U54,
+    55 => ::typenum::U55,
+    56 => ::typenum::U56,
+    57 => ::typenum::U57,
+    58 => ::typenum::U58,
+    59 => ::typenum::U59,
+    60 => ::typenum::U60,
+    61 => ::typenum::U61,
+    62 => ::typenum::U62,
+    63 => ::typenum::U63,
+    64 => ::typenum::U64,
+
+    70 => ::typenum::U70,
+    80 => ::typenum::U80,
+    90 => ::typenum::U90,
+
+    100 => ::typenum::U100,
+    200 => ::typenum::U200,
+    300 => ::typenum::U300,
+    400 => ::typenum::U400,
+    500 => ::typenum::U500,
+
+    128 => ::typenum::U128,
+    256 => ::typenum::U256,
+    512 => ::typenum::U512,
+
+    1000 => ::typenum::U1000,
+    1024 => ::typenum::U1024
+}
